const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const MIN_REFRESH_INTERVAL = 5000;

var _logger = Cc['@flock.com/logger;1'].createInstance(Ci.flockILogger);
_logger.init("myworld");

var gMetrics = Cc["@flock.com/metrics-service;1"].getService(Ci.flockIMetricsService);

var _coop = Cc["@flock.com/singleton;1"]
  .getService(Ci.flockISingleton)
  .getSingleton("chrome://browser/content/flock/common/load-faves-coop.js")
  .wrappedJSObject;

const BLOG_SVC = Cc["@flock.com/flock-blog;1"].getService(Ci.flockIBlogService);

const PREFS = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
const PREF_BROWSER_STARTUP_HOMEPAGE = "browser.startup.homepage";
const PREF_MYWORLD_CURRENT_ENGINE   = "flock.myworld.currentEngine";
const XMLHTTPREQUEST_CONTRACTID     = "@mozilla.org/xmlextras/xmlhttprequest;1";
const nsIXMLHttpRequest             = Components.interfaces.nsIXMLHttpRequest;

const nsIPrefLocalizedString = Components.interfaces.nsIPrefLocalizedString;

const RDFS = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);

function Namespace(ns) { return function (arg) { return RDFS.GetResource(ns+arg); } }
const FLRDF  = Namespace("http://flock.com/rdf#");
const NCRDF  = Namespace("http://home.netscape.com/NC-rdf#");
const kXULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var rdfLiteral = RDFS.GetLiteral;

var currentEng;
var gLastRefresh = new Date();
var flaggedForRefresh = false;

window.addEventListener("resize", resizeFunc, false);
function resizeFunc(event) {
  doResize();
}

var gFavObs = null;
var gObsrvr = null;

var feedTemplateObs = {
  willRebuild: function(aBuilder) {
    
  },
  didRebuild: function(aBuilder) {
    // JMC: If we don't have any content, show the "empty" message.
    // Count non-rule and non-template children
    var hasUnseenFeeds = false;
    var feedsWrapperRoot = document.getElementById('feedsWrapperBox');
    for (var i=0; i < feedsWrapperRoot.childNodes.length; i++) {
      if (feedsWrapperRoot.childNodes[i].tagName == 'feedstream') {
        hasUnseenFeeds = true;
      }
    }
    document.getElementById('unreadID').setAttribute('hidden', hasUnseenFeeds);
  }
};

function enableDisableObservers(bValue) {
  var rdfsService = RDFS.QueryInterface(Ci.nsIRDFService);
  var dataSource = rdfsService.GetDataSource("rdf:flock-favorites");
  if(bValue) {
    // Enable observers
  
    var observable = dataSource;
    observable.QueryInterface(Ci.flockIRDFObservable);
    observable.addArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, FLRDF("photoid"), null, gFavObs);
    observable.addArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, FLRDF("cachedThumbnail"), null, gFavObs);
    observable.addArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, FLRDF("unseenItems"), null, gFavObs);
    observable.addArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, NCRDF("bookmark"), null, gFavObs);
    observable.addArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, NCRDF("isPollable"), null, gFavObs);
  } else {
    // Disable observers
    var observable = dataSource;
    observable.QueryInterface(Ci.flockIRDFObservable);
    observable.removeArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, FLRDF("photoid"), null, gFavObs);
    observable.removeArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, FLRDF("cachedThumbnail"), null, gFavObs);
    observable.removeArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, FLRDF("unseenItems"), null, gFavObs);
    observable.removeArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, NCRDF("bookmark"), null, gFavObs);
    observable.removeArcObserver(Ci.flockIRDFObserver.WATCH_TYPES, null, NCRDF("isPollable"), null, gFavObs);
  } 
}

function onUnload() {
  var feedsRoot = document.getElementById('feedBoxContents');
  feedsRoot.builder.removeListener(feedTemplateObs);

  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('myworld close');

  var os = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
  os.removeObserver(gObsrvr, "browser-search-engine-modified");
  os.removeObserver(gObsrvr, "refresh-myworld");
  os.removeObserver(gObsrvr, "refresh-myworld-topsites");
  os.removeObserver(gObsrvr, "refresh-myworld-accounts");
  
  enableDisableObservers(false);
}
function onLoad()
{
  _logger.debug("onLoad");

  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('myworld open');
  
  doResize();
   
  checkDefaultHomepage();  
  
  loadEngines();

  refreshAccounts();
  setTimeout("refreshDateTime()", 500);
 

  // Add observer for refresh
  var inst = this;
  gFavObs = {};
  gFavObs.rdfChanged = function (ds, type, source, predicate, target, oldtarget) {
    if (!source) return;
   
    var coopObj = _coop.get_from_resource(source);
    if (!coopObj) return;
    if(coopObj.isInstanceOf(_coop.RichPhoto)) {
        if (predicate.Value == "http://flock.com/rdf#cachedThumbnail" &&
            target.Value != "")  {
          var obsService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
          obsService.notifyObservers(null, "refresh-myworld", null);
        }
        return;
    } else if(coopObj.flockType == "bookmark" && coopObj.photoid) {  
      // early exit, don't resize since we still have the same amount of content.
      return;
    } else if(coopObj.flockType == "Account") {
      setTimeout("refreshAccounts()", 500);
    }
    
  }
  
  gObsrvr = {};
  gObsrvr.observe = function engineManager_observer(aEngine, aTopic, aVerb) {
    if(aTopic == "browser-search-engine-modified") {
      aEngine.QueryInterface(Ci.nsISearchEngine);
      switch( aVerb) {
        case "engine-added":
        case "engine-removed":
          setTimeout("loadEngines()", 1000);
      }
    } else if (aTopic == "refresh-myworld") {
      var now = new Date();
      
      var deltaTime = now - gLastRefresh;
      
      if(deltaTime < MIN_REFRESH_INTERVAL && !inst.flaggedForRefresh) {
        // Fire it off in X seconds
        setTimeout("refreshMedia()", MIN_REFRESH_INTERVAL - deltaTime);
        inst.flaggedForRefresh = true;
      } else if (!inst.flaggedForRefresh) {
        // Do it now.
        setTimeout("refreshMedia()", 500);
        inst.flaggedForRefresh = true;
      }
    } else if (aTopic == "refresh-myworld-topsites") {
      setTimeout("decorateTopSites()", 500);
    } else if (aTopic == "refresh-myworld-accounts"){
      setTimeout("refreshAccounts()", 500);
    }
  }
  // Add observer for engines added
  var os = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
  os.addObserver(gObsrvr, "browser-search-engine-modified", false);
  os.addObserver(gObsrvr, "refresh-myworld", false);
  os.addObserver(gObsrvr, "refresh-myworld-topsites", false);
  os.addObserver(gObsrvr, "refresh-myworld-accounts", false);

  var feedsRoot = document.getElementById('feedBoxContents');
  feedsRoot.builder.addListener(feedTemplateObs);
  feedsRoot.builder.rebuild();
  
  setTimeout("decorateTopSites()", 500);

  enableDisableObservers(true);
}

function decorateTopSites() {
  // Simply go through the entire topsites setting odd/even styling.
  var topSites = document.getElementById("faveContent");
  
  var children = topSites.childNodes;
  for(var i=1;i<children.length;i++) {
    var topsite = children[i];
    if(i % 2) {
      topsite.setAttribute("class", "faveSite"); 
    } else {
      topsite.setAttribute("class", "faveSiteOdd"); 
    }
  }
  
}

function doResize() {
  
  theHeight = window.innerHeight - 350;
  if(theHeight < 290)
    theHeight = 290;

  var ftc = document.getElementById("ftcontainerID");
  var fc = document.getElementById("faveContent");
  var fb = document.getElementById("feedBoxContents");
  var mb = document.getElementById("mediaBoxContents");
  
  var oldfcHeight = fc.getAttribute("height");
  var oldfbHeight = fb.getAttribute("height");
  var oldmbHeight = mb.getAttribute("height");
  
  if(oldfcHeight != theHeight
      || oldfbHeight != theHeight
      || oldmbHeight != theHeight) {
    fc.setAttribute("height", theHeight + "px");
    fb.setAttribute("height", theHeight + "px");
    mb.setAttribute("height", theHeight + "px");
  }
}

function checkDefaultHomepage() {
   var defhbox = document.getElementById("makedefault");
   
   // Remove to re-build
   while(defhbox.firstChild) {
      defhbox.removeChild(defhbox.firstChild); 
   }
   
   // check if flock is in the default
   if(PREFS.getPrefType(PREF_BROWSER_STARTUP_HOMEPAGE)) {
      var homepage = PREFS.getComplexValue(PREF_BROWSER_STARTUP_HOMEPAGE, nsIPrefLocalizedString).data;
      if(homepage.indexOf("about:myworld") >= 0) {
        return; 
      }
   }
   
   // Add to page if nescessary
   var button = document.createElement("button");
   button.setAttribute("onclick", "makeDefaultHomepage()");
   button.setAttribute("class", "defaultHomepageBtn");
   
   var label = document.createElement("label");
   label.setAttribute("class", "myworldHomepage");
   label.setAttribute("value", flockGetString("start/myworld", "flock.myworld.homepage.set"));
   
   button.appendChild(label);
   defhbox.appendChild(button);
}

function loadEngines() {
  // Get a list of all search engines
  var searchService = Cc["@mozilla.org/browser/search-service;1"].getService(Ci.nsIBrowserSearchService);
  if (!searchService) return;
  
  var engines = searchService.getVisibleEngines({ });

  var eMenu = document.getElementById("engineMenuID");

  // delete list before re-creating it
  var existingChildrenCount = eMenu.childNodes.length;
  for(var z=existingChildrenCount;z>0;z--)
  {
    var existChild = eMenu.childNodes[z-1];
    eMenu.removeChild(existChild);
  }

  var currentEngine;
  // Get current myworld engine
  if(PREFS.getPrefType(PREF_MYWORLD_CURRENT_ENGINE)) {
    currentEngine = PREFS.getCharPref(PREF_MYWORLD_CURRENT_ENGINE);
  } else {
    // If myworld engine doesnt exist, use current default engine
    if (!searchService.currentEngine) return;
    currentEngine = searchService.currentEngine.name;
  }

  PREFS.setCharPref(PREF_MYWORLD_CURRENT_ENGINE, currentEngine);
  
    // Add them to the menu
  for(var i=engines.length-1;i>=0;--i) {
    var menuitem = document.createElementNS(kXULNS, "menuitem");
    menuitem.setAttribute("label", engines[i].name);
    menuitem.setAttribute("id", engines[i].name);
    menuitem.setAttribute("class", "engine-menuitem");
    menuitem.setAttribute("engine", engines[i].name);
    menuitem.setAttribute("image", engines[i].iconURI);
    menuitem.setAttribute("oncommand", "selectEngine(this)");

    if(engines[i].name == currentEngine) {
      menuitem.setAttribute("selected", "true");
      currentEng = engines[i];
    }

    if(engines[i].iconURI) {
      menuitem.setAttribute("image", engines[i].iconURI.spec);

      // Also set the image of the selected item
      if(engines[i].name == currentEngine) {
        var engineImageID = document.getElementById("engineImageID");
        engineImageID.setAttribute("src",engines[i].iconURI.spec);
      }
    }

    eMenu.insertBefore(menuitem, eMenu.firstChild);
    menuitem.engine = engines[i];
  }
}

function toggleEngineMenu() {
  var popup = document.getElementById("engineMenuID");
  var popButton = document.getElementById("popButton");

  popup.showPopup(popButton, -1, -1, "popup", "bottomleft", "topleft");
}

function selectEngine(engine) {
  var engineName  = engine.getAttribute("engine");
  var engineImage = engine.getAttribute("image");
  gMetrics.reportCount("myworld select search "+engineName);

  if(engineImage != "" && engineImage != null) {
    var engineImageID = document.getElementById("engineImageID");
    engineImageID.setAttribute("src",engineImage);
  }

  // Set current engine
  var searchService = Cc["@mozilla.org/browser/search-service;1"].getService(Ci.nsIBrowserSearchService);
  var engines = searchService.getVisibleEngines({ });

  for(var i=engines.length-1;i>=0;--i) {
    if(engines[i].name == engineName) {
      currentEng = engines[i];
      PREFS.setCharPref(PREF_MYWORLD_CURRENT_ENGINE, currentEng.name);
      return;
    }
  }
}

function openFeedHeadline(headlineURL, headlineID, event) {
  if(event.button == 2) {
    return;
  }
  
  gMetrics.reportCount("myworld feed headline click");
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    //win.newsSidebarOpen();
    openURLMyWorld(headlineURL, event, false);
    
    var coopObj = _coop.get(headlineID);
    coopObj.unseen = false;
  }

}

function populateTooltip(node) {

  var headlines = node.getAttribute("headlines");
  var headlinesArr = headlines.split("|||");

  var tipContainer = document.getElementById("tipcontainer");

  while(tipContainer.firstChild) {
    tipContainer.removeChild(tipContainer.firstChild);
  }

  for(var i=0;i<headlinesArr.length;i++) {
    var headline = headlinesArr[i];

    var label = document.createElement("label");
    label.setAttribute("value", headline);

    tipContainer.appendChild(label);
  }
}

function getFeedHeadlines(feed) {
  var headlines = [];

  var childrenCount = feed.childNodes.length;

  var z=0;

  for(var i=childrenCount;i> 0;i--) {

    if(i < 2)
      return headlines;
      
    // Only record unread headlines
    var coopObj = _coop.get(feed.childNodes[i-1].id);
    if(coopObj && !coopObj.unseen)
      continue;

    headlines[z]    = [];
    headlines[z][0] = feed.childNodes[i-1].firstChild.getAttribute("feedDescription");
    headlines[z][1] = feed.childNodes[i-1].firstChild.getAttribute("headlineURL");
    headlines[z][2] = feed.childNodes[i-1].id;

    // return a maximum of 3 headlines
    if(z++ == 2)
      return headlines;
  }

  return headlines;
}

function setOpacity(node, opacityNum) {
  node.style.opacity = opacityNum;
}

function refreshAccounts()
{
  var bFoundAvatar = false;

  var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
  var e = catMgr.enumerateCategory("flockWebService");
  var serviceBox = document.getElementById("availableServices");
  var svcChild = serviceBox.firstChild;
  while (svcChild) {
    serviceBox.removeChild(svcChild);
    svcChild = serviceBox.firstChild;
  }
  var foundAccounts = false;
  while (e.hasMoreElements()) {
    var entry = e.getNext().QueryInterface(Ci.nsISupportsCString);
    if (!entry) continue;
    var contractID = catMgr.getCategoryEntry("flockWebService", entry.data);
    var svc = Cc[contractID].getService(Ci.flockIWebService);
    if (svcHasAccounts(svc)) {
      // Set Avatar - pick the very first valid loc that we can find
      if (!bFoundAvatar) {
        var avatarImg = document.getElementById("avatarImgId");

        // remove old avatar if it's there
        var oldAvatarImg = avatarImg.firstChild;
        while(oldAvatarImg) {
          avatarImg.removeChild(oldAvatarImg);
          oldAvatarImg = avatarImg.firstChild;
        }

        // Get avatar img location
        var imgLoc = getAvatarImgSrc(svc);
        if(imgLoc != ""
           && imgLoc != null) {
          // Set new avatar
          var img = document.createElement("image");
          img.setAttribute("src", imgLoc );
          img.setAttribute("class", "avatarimgclass");
          avatarImg.appendChild(img);

          bFoundAvatar = true;
        }
      }

      foundAccounts = true;
    } else {
      // Check for ignored services
      var ignoreEntry;
      PREFS.getPrefType("flock.accounts.ignore") && (ignoreEntry =
        PREFS.getCharPref("flock.accounts.ignore"));

      // Check to see if we should NOT show this in the list
      var found = false;
      if (ignoreEntry) {
        var ignoreList = ignoreEntry.split(",");
        for each (var ignore in ignoreList) {
          if (svc.shortName.toLowerCase() == ignore.toLowerCase()) {
            found = true;
            break;
          }
        }
      }
      // Did we find something?
      if (found) {
        continue;
      }

      var stack = document.createElement("stack");
      /* BC: We're just using css to create this bg for now, may change later so keep this code.
      var bg = document.createElement("image");
      bg.setAttribute("src", "chrome://browser/skin/myworld/favicon-border.png");
      stack.appendChild(bg);
      */
      stack.setAttribute("class","favStack");
      var fg = document.createElement("image");
      fg.setAttribute("src", svc.icon);
      fg.setAttribute("urn", svc.urn);
      fg.setAttribute("top", "1");
      fg.setAttribute("left", "1");
      fg.setAttribute("serviceName", svc.shortName); // XXX
      fg.setAttribute("tooltiptext", svc.title);
      fg.setAttribute("onclick", "openService(this, event)");
      fg.setAttribute("class", "accountIcon");
      stack.appendChild(fg);
      serviceBox.appendChild(stack);
    }
  }
  var accountsArea = document.getElementById("configuredAccounts");
  var avatarContainer = document.getElementById("avatarContainer");
  
  if(bFoundAvatar) {
    avatarContainer.removeAttribute("hidden");
  } else {
    avatarContainer.setAttribute("hidden", "true");
  }
  
  if (foundAccounts) {
    accountsArea.removeAttribute("hidden");
  } else {
    accountsArea.setAttribute("hidden", "true");
  }
}

function getUpdatedDate(lastUpdated) {
  var date = lastUpdated.split(" ")[0];

  var m = date.split("/")[1];
  var d = date.split("/")[0];
  var y = date.split("/")[2];

  date = new Date(lastUpdated);
  var datenow = new Date();

//@line 541 "/home/build/tinderbuild/src/flock/mozilla/browser/components/flock/start/content/myworld.js"
  date.setMonth(m-1);
  date.setDate(d);
  date.setFullYear("20" + y);
//@line 550 "/home/build/tinderbuild/src/flock/mozilla/browser/components/flock/start/content/myworld.js"

  var updatedDate = datenow.getTime() - date.getTime();

  return updatedDate;
}

function getFriendlyLastDate(lastUpdated) {
  var updatedDate = getUpdatedDate(lastUpdated);

  if (lastUpdated == -1) {
    // Invalid date, windows only.
    return "";
  }

  var minutesSince = ((updatedDate/1000) / 60);
  var hoursSince = ((updatedDate/1000) / 60) / 60;
  var daysSince = ((updatedDate/1000) / 60) / 60 / 24;

  if(daysSince >= 2.0) {
    return flockGetString("start/myworld", "flock.myworld.daysAgo", [daysSince.toFixed(0)]);
  } else if (daysSince >= 1.0) {
    return flockGetString("start/myworld", "flock.myworld.dayAgo", [1]);
  } else if (hoursSince >= 2.0){
    return flockGetString("start/myworld", "flock.myworld.hrsAgo", [hoursSince.toFixed(0)]);
  } else if (hoursSince >= 1.0){
    return flockGetString("start/myworld", "flock.myworld.hrAgo", [1]);
  } else if (minutesSince >= 2.0) {
    return flockGetString("start/myworld", "flock.myworld.minsAgo", [minutesSince.toFixed(0)]);
  }

  // Invalid value check
  if (minutesSince < 0.0 ) {
    // Under/Overflow
    return "";
  }

  return flockGetString("start/myworld", "flock.myworld.momentAgo");
}

function openService(node, event)
{
  if(event.button == 2) {
    return;
  }

  // Open the service URL in a new tab
  var svcName = node.getAttribute("serviceName");
  gMetrics.reportCount("myworld service click "+svcName);
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    win.openUILinkIn("http://redir.flock.com/redir?component=browser&action=myworld_click&partner=" + svcName, "tab");
  }

  // Tell the Web Service Manager to automatically keep any accounts
  // created for this service in the next while...
  var wsm = Cc["@flock.com/webservice-manager;1"].getService(Ci.flockIWebServiceManager);
  wsm.autoKeepAccountForService(node.getAttribute("urn"));
}

function getAvatarImgSrc(svc) {
  var accounts = svc.getAccounts();
  if (!accounts) return "";
  accounts.QueryInterface(Ci.nsISimpleEnumerator);

  while (accounts.hasMoreElements()) {
    var account = accounts.getNext();

    account.QueryInterface(Ci.flockIWebServiceAccount);

    var faves_coop = Cc['@flock.com/singleton;1'].getService(Ci.flockISingleton).getSingleton('chrome://browser/content/flock/common/load-faves-coop.js').wrappedJSObject;

    var acc = faves_coop.get(account.urn);

    if(acc) {
      return acc.avatar;
    }
  }

  return "";
}

function refreshDateTime()
{
  var timeLabel = document.getElementById("timeUpdated");
  var dateLabel = document.getElementById("dateUpdated");
  var time = new Date();
  var hours = time.getHours();
  var mins = time.getMinutes();
  if (mins < 10) mins = "0"+mins;
  if (hours >= 12) {
    var ampm = flockGetString('start/myworld', 'pm-string');;
    if (hours > 12) hours -= 12;
  } else {
    var ampm = flockGetString('start/myworld', 'am-string');;
  }
  
  var day = flockGetString('start/myworld', 'day.'+(time.getDay()+1)+'.name');
  var month = flockGetString('start/myworld', 'month.'+(time.getMonth()+1)+'.name');
  var thedate = day + ", " + month + " " + time.getDate() + ", " + time.getFullYear();
  
  var timeString = hours+":"+mins+ampm.toLowerCase();
  timeLabel.setAttribute("value", timeString);
  dateLabel.setAttribute("value", thedate);

  setTimeout("refreshDateTime()", (60 - time.getSeconds() + 1) * 1000);
}

function svcHasAccounts(svc)
{
  svc.QueryInterface(Ci.flockIWebService);
  var accounts = svc.getAccounts();
  if (!accounts) return false;
  accounts.QueryInterface(Ci.nsISimpleEnumerator);
  return accounts.hasMoreElements();
}

function reloadMyWorld() {
  // XXX: unfortunately the template does not re-sort the list until a page reload occurrs.
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    var gBrowser = win.gBrowser;
    
    for(var i=0;i<gBrowser.mTabs.length;i++) {
      var browser = gBrowser.mTabs[i].linkedBrowser;
       if(browser.currentURI.spec == "about:myworld") {
         browser.reload();
       }
    }
  }
}

function refreshMedia()
{
  // force redraw of template area.
  var box = document.getElementById("mediaBoxContents");
  box.builder.rebuild();
  gLastRefresh = new Date();
  flaggedForRefresh = false;
}

function uploadPhotos(event)
{
  if(event.button == 2) {
    return; 
  }
  
  gMetrics.reportCount("myworld open uploader");
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    win.flock_photo.util.launchUploader();
  }
}

function createBlogPost(event)
{
  if(event.button == 2) {
    return; 
  }
  
  gMetrics.reportCount("myworld open blogpost");
  BLOG_SVC.openEditor("","","");
}

function searchboxKeyPress(event,query) {
  if (event.keyCode == KeyEvent.DOM_VK_RETURN) {
    searchFunction(query);
  }
}

function searchFunction(query)
{
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    var ss = Cc["@mozilla.org/browser/search-service;1"].getService(Ci.nsIBrowserSearchService);
    var submission = currentEng.getSubmission(query, null); // HTML response
    if (!submission) { return; }

    gMetrics.reportCount("myworld search "+currentEng.name);
    var submissionURL = tweakAffiliateCode(submission.uri);
    win.openUILinkIn(submissionURL, "tab");
  }
}

function tweakAffiliateCode(aSearchURI)
{
  var spec = aSearchURI.spec;
  // If this is a Yahoo! search, we want to tweak the affiliate search code so
  // we know that this search is being done from My World
  if (aSearchURI.host == "search.yahoo.com") {
    // This particular regex trick relies on the fact that My World types are
    // the same as the chrome types, except My World types have a "w" appended
    // to the type's letters.  (ex: fl09 becomes flw09)
    spec = spec.replace(/&type=([a-z]+)?09/, "&type=$1w09");
  }
  return spec;
}

function openWebClipboard(event) {
  if(event.button == 2) {
    return; 
  }  
  
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    // Only open, don't close if already open
    win.toggleSidebar("flock_notesSidebarBroadcaster", true);
  }
}

function openAASSidebar(node, event)
{
  if(event.button == 2) {
    return;
  }
  
  if(node == null) {
    // Just open the sidebar.
    var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
    var win = wm.getMostRecentWindow('navigator:browser');
    if (win) {
      gMetrics.reportCount("myworld accounts");
      // Only open, don't close if already open
      win.toggleSidebar("flock_AccountsSidebarBroadcaster", true);
    }
    
    return; 
  }
 
  var homeURL = node.getAttribute("homeURL");
  openURLMyWorld(homeURL, event, false);
}

function openFavoritesManager(event)
{
  if(event.button == 2) {
    return;
  }

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    // Only open, don't close if already open
    win.toggleSidebar("viewBookmarksSidebar", true);
  }
}

function openNewsSidebar(node, event)
{
  if(event.button == 2) {
    return;
  }
 
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    win.newsSidebarOpen();

    if(node != null) {
      gMetrics.reportCount("myworld view feed");
      var url = encodeURIComponent(node.getAttribute("url"));
      openURLMyWorld("flock://favorites/urn:flock:feed:" + url,event, false);
    }
  }
}


function openImage(node, event) {
  if(event.button == 2) {
    return;
  }
 
  gMetrics.reportCount("myworld photo click");
  var imageURL = node.getAttribute("imageURL");
  
  var dataNode = node.parentNode;

    // for Truveo videos, always open in new window
  // c.f. http://bugzilla.flock.com/show_bug.cgi?id=7045
  if(dataNode.getAttribute("svc") == "searchvideo") {
    openURLMyWorld(imageURL, event, true);
  } else {
    openURLMyWorld(imageURL, event, false);

    // Open the media bar
    openMediaBar(dataNode, event);
  }
}

function openMediaBar(node, event)
{
  if(event.button == 2) {
    return;
  }
  
  gMetrics.reportCount("myworld view mediastream");
  var service = node.getAttribute("svc");
  var query   = node.getAttribute("query");
  var name    = node.getAttribute("name");

  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    win.flock_topbarSelectById('flock_MediabarTopbarBroadcaster',true);
    
    if(service != "") {
      win.mediabar.loadQuery(service,query,name);
    }
  }
}

function openURL(node, event)
{
  if(event.button == 2) {
    return;
  }
 
  var url = node.childNodes[1].childNodes[1].value;
  gMetrics.reportCount("myworld fave click");

  // Open in new tab
  openURLMyWorld(url, event, false);
}

function popupWhatsThis(node) {
  var description   = node.getAttribute("desc");
  var descriptionID = node.getAttribute("descID");

  var popup = document.getElementById(descriptionID);

  while(popup.firstChild) {
    popup.removeChild(popup.firstChild);
  }

  var descNode = document.createElement("description");
  descNode.setAttribute("class", "whatsThisPopup");
  descNode.setAttribute("flex", "1");
  var text = document.createTextNode(description);
  descNode.appendChild(text);
  popup.appendChild(descNode);

  popup.showPopup(node, -1, -1, "popup", "bottomleft", "topleft");
}

function makeDefaultHomepage() {
  // Make My World the default homepage
  PREFS.setCharPref(PREF_BROWSER_STARTUP_HOMEPAGE, "about:myworld");
  
  checkDefaultHomepage();
 
  var notificationBox = getNotificationBox(); 
  if(notificationBox) {
    var message = flockGetString("start/myworld", "flock.myworld.homepage");
    notificationBox.appendNotification( message,
                                        "myworldadded",
                                        "chrome://browser/skin/Info.png", 
                                        notificationBox.FLOCK_PRIORITY_MEDIUM,
                                        null,
                                        null);
  
  }
}

function getNotificationBox() {
  
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    var gBrowser = win.getBrowser();
    var notificationBox = gBrowser.getNotificationBox();                                     
    
    return notificationBox;
  } 
  
  return null;
}

function openURLMyWorld(url, event, forceWindow) {
  _logger.debug("myworld open event btn" + event.button);

  if(event.button == 2) {
    return;
  }
  
  var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var win = wm.getMostRecentWindow('navigator:browser');
  if (win) {
    /* if(event.metaKey || event.ctrlKey || event.button == 1) {
      // opens url in new tab in the background - allows you to "queue" up tabs for viewing
      var gBrowser = win.getBrowser();
      gBrowser.addTab(url, null, null, null, null, null);
    } else */ 
    if (forceWindow) {
      var options = "centerscreen,resizable=yes";
//@line 951 "/home/build/tinderbuild/src/flock/mozilla/browser/components/flock/start/content/myworld.js"
      win.open(url, "miniView", options);
    } else {
      // opens url in new tab and switches to it immediately
      // win.openUILinkIn(url,"tab");
      
      switch (win.whereToOpenLink(event)) {
        case "current":
          win.openUILinkIn(url, "tab");
          break;
        case "window":
          win.getTopWin().openNewWindowWith(url, null, null);
          break;
        default: // "tab" or "tabshifted"
          win.getTopWin().openNewTabWith(url, null, null, event);
          break;
      }
    } 
  }
}

window.onload = onLoad;
window.onunload = onUnload;
